/*
 * Read from Android logger.
 * Author: Peter Wu <peter@lekensteyn.nl>
 * Date: 2014-06-22
 */
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>

#define __LOGGERIO          0xAE
#define LOGGER_GET_VERSION  _IO(__LOGGERIO, 5) /* abi version */
#define LOGGER_SET_VERSION  _IO(__LOGGERIO, 6) /* abi version */


static int get_version(int fd) {
    int r;

    r = ioctl(fd, LOGGER_GET_VERSION);
    if (r < 0) {
        perror("ioctl(LOGGER_GET_VERSION)");
    } else {
        fprintf(stderr, "Logger version: %i\n", r);
    }

    return r;
}

static int set_version(int fd, int ver) {
    int r;

    r = ioctl(fd, LOGGER_SET_VERSION, &ver);
    if (r < 0) {
        perror("ioctl(LOGGER_SET_VERSION)");
    } else {
        fprintf(stderr, "Version set to %i (ret=%i)\n", ver, r);
    }

    return r;
}

int main(int argc, char **argv) {
    int fd, r;
    char buf[1024 * 5 + 1];
    int version = -1;

    if (argc < 2) {
        fprintf(stderr, "Usage: %s /dev/log/... [version]\n", argv[0]);
        return 1;
    }
    if (argc > 2) {
        version = atoi(argv[2]);
        if (version < 1 || version > 2) {
            fprintf(stderr, "Version must be 1 or 2\n");
            return 1;
        }
    }

    fd = open(argv[1], O_RDONLY | O_NONBLOCK);
    if (fd < 0) {
        perror(argv[1]);
        return 1;
    }

    get_version(fd);
    if (version > 0) {
        if (set_version(fd, version) < 0) {
            close(fd);
            return 1;
        }
        get_version(fd);
    }

    do {
        r = read(fd, buf, sizeof(buf));
        if (r > 0) {
            write(STDOUT_FILENO, buf, r);
        }
    } while (r > 0);

    close(fd);
    return 0;
}
